'use strict';

var Promise = require('bluebird');
var _ = require('lodash');

function RemoteApps(options) {
  this.__options = options;
}

RemoteApps.prototype.init = function() {
  return Promise.resolve();
};

RemoteApps.resolve = function(options) {
  var cliConfig = require('apiconnect-cli-config');
  var config = cliConfig.loadConfig();
  var appConfig = config.getOne('app') || { data: {} };
  options.serviceFromConfig = !options.service;
  _.defaults(options, {
    server: appConfig.hostname || '',
    organization: appConfig.data.orgs,
    service: appConfig.data.apps,
  });
  var cliUtil = require('../../index');
  var providerOrgPromise = cliUtil.Orgs.findOrgByNameAsync(
    options.server, options.organization
  );
  var providerAppPromise;
  if (options.service) {
    providerAppPromise = providerOrgPromise.then(function(providerOrg) {
      return cliUtil.ProviderApps.get({
        server: options.server,
        org: providerOrg.id,
        app: options.service,
        getEncrypted: true,
      });
    });
  } else {
    providerAppPromise = Promise.resolve(null);
  }
  return Promise.all([ providerOrgPromise, providerAppPromise ]).then(function(info) {
    options.providerApp = info[1] || {};
    options.providerOrg = info[0];
    if (options.providerApp.bluemix) {
      var Bluemix = require('../bluemix');
      return new Bluemix(options);
    } else if (options.providerApp.liberty) {
      var LibertySupport = require('../collectives');
      return new LibertySupport(options);
    } else {
      throw new Error('Unsupported application runtime');
    }
  }).then(function(remote) {
    return remote.init().then(function() {
      return remote;
    });
  });
};

module.exports = RemoteApps;
