// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

/********************************************************* {COPYRIGHT-TOP} ***
 * IBM Confidential
 * OCO Source Materials
 * 5725-C11, 5725-C12, 7198-8FX, 5725-F70, 5725-F71, 5725-F72, 5725-L30
 *
 * (C) Copyright IBM Corporation 2015  All Rights Reserved.
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 ********************************************************* {COPYRIGHT-END} **/

var _ = require('lodash');
var f = require('util').format;
var Promise = require('bluebird');
var request = require('request-promise');
var apicConfig = require('apiconnect-cli-config');
var apicUtil = require('./util');

var config = apicConfig.loadConfig();

function getMetadata(serverConfig, opts) {

  var options = _.assign({
    forceRefresh: false,
  }, opts);

  var configKey = serverConfig.hostname + '.meta';

  var serverMeta = config.getOne(configKey);

  if (serverMeta && !options.forceRefresh) {
    return Promise.resolve(new ServerMeta(serverMeta));
  }

  var requestOpts = apicUtil.requestOptions(null, null, serverConfig.hostname);
  requestOpts.url += '/v1/cloud/metadata/apimanager';
  requestOpts.method = 'get';
  requestOpts.json = true;
  return request(requestOpts).then(function(body) {

    if (!body[0]) { throw new Error(f('Metadata for [%s] could not be loaded', serverConfig.hostname)); }

    serverMeta = {
      formFactor: body[0].formFactor,
      serverCapabilities: body[0].serverCapabilities || {},
    };

    var confEntry = {};
    confEntry[configKey] = serverMeta;

    config.set(confEntry);
    return new ServerMeta(serverMeta);
  });

}

function ServerMeta(meta) {
  this.meta = meta;
}

ServerMeta.prototype.supports = function(key, expectedValue) {

  var capabilities = this.meta.serverCapabilities;

  if (!capabilities[key]) { return false; }

  if (Array.isArray(capabilities[key])) {
    return capabilities[key].indexOf(expectedValue) > -1;
  }

  return capabilities[key] === expectedValue;

};

ServerMeta.prototype.isBluemix = function() {
  return this.meta.formFactor.indexOf('BLUEMIX') > -1;
};

module.exports = {
  getMetadata: getMetadata,
};
